package cn.fastfun.service.impl;

import cn.fastfun.controller.dto.CellDto;
import cn.fastfun.controller.param.QueryParamForCellModel;
import cn.fastfun.service.CellModelService;
import cn.fastfun.service.UtilService;
import cn.fastfun.service.entity.CellModel;
import cn.fastfun.service.vo.CellModelVo;
import com.bridge.dto.ApiPageDTO;
import com.bridge.dto.QueryParam;
import com.bridge.dto.QueryParamExp;
import com.bridge.service.impl.JpaServiceImp;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.persistence.Query;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.regex.Pattern;

/**
 * @author user
 */
@Slf4j
@Service
public class CellModelServiceImpl extends JpaServiceImp<CellModel, String> implements CellModelService {

    private static final String OK = "OK";

    @Resource
    UtilService utilService;

    /**
     * 分页搜索单体型号
     *
     * @param param 参数
     * @return com.bridge.dto.ApiPageDTO
     */
    @Override
    public ApiPageDTO getCellModelPageInfo(QueryParamForCellModel param) {
//        QueryParam finalQuery = new QueryParam();
        if (StringUtils.isNotBlank(param.getCellModel())) {
            param.addParam(QueryParamExp.eq("cell_model", param.getCellModel()));
        }
        Page<CellModel> page = this.findByParam(param);
        ApiPageDTO result = new ApiPageDTO("搜索成功！", page);
        return result;
    }

    /**
     * 单体型号详情
     *
     * @param id 参数
     * @return cn.fastfun.service.vo.CellModelVo
     */
    @Override
    public CellModelVo getCellDetail(String id) {
        CellModelVo vo = new CellModelVo();
        CellModel cellModel = this.getOne(QueryParamExp.eq("id", id));
        BeanUtils.copyProperties(cellModel, vo);
        return vo;
    }

    /**
     * 删除单体型号
     *
     * @param id 参数
     * @return java.lang.String
     */
    @Override
    public String delCell(String id) {
        CellModel cellModel = this.get(id);
        //校验包型号是否被使用
        String res = checkCellModelUsed(cellModel.getCellModel());
        if (!res.equals(OK)) {
            return res;
        }
        this.delete(id);
        return OK;
    }

    /**
     * 新增单体型号
     *
     * @param dto 参数
     * @return java.lang.String
     */
    @Override
    public String addCell(CellDto dto) {
        //校验包型号是否被使用
        String res = checkCellModelUsed(dto.getCellModel());
        if (!res.equals(OK)) {
            return res;
        }
        //校验包型号是否被使用
        res = checkModelRule(dto.getCellModel());
        if (!res.equals(OK)) {
            return res;
        }
        //校验单体型号是否已经存在
        res = checkCellModelExist(dto);
        if (!res.equals(OK)) {
            return res;
        }
        CellModel cellModel = new CellModel();
        BeanUtils.copyProperties(dto, cellModel);
        cellModel.setOperator(utilService.getUserName());
        this.save(cellModel);
        return OK;
    }

    /**
     * 修改单体型号
     *
     * @param dto 参数
     * @return java.lang.String
     */
    @Override
    public String modifyCell(CellDto dto) {
        CellModel cellModel = this.get(dto.getId());
        //校验包型号规则
        String res = checkModelRule(dto.getCellModel());
        if (!res.equals(OK)) {
            return res;
        }
        //校验包型号是否被使用
        res = checkCellModelUsed(cellModel.getCellModel());
        if (!res.equals(OK)) {
            return res;
        }
        //校验包型号是否已经存在
        res = checkCellModelExist(dto);
        if (!res.equals(OK)) {
            return res;
        }
        BeanUtils.copyProperties(dto, cellModel);
        cellModel.setOperator(utilService.getUserName());
        this.save(cellModel);
        return OK;
    }

    /**
     * 校验包型号是否被使用
     *
     * @param model 参数
     * @return java.lang.String
     */
    private String checkCellModelUsed(String model) {
        StringBuilder builder = new StringBuilder();
        builder.append("select count(0) from app_device where cell_model='").append(model).append("' ");
        log.info("SQL: {}", builder.toString());
        Query q = this.getQueryBySQL(builder.toString(), new HashMap<>());
        long amount = ((BigInteger) q.getSingleResult()).longValue();
        if (amount > 0) {
            return "单体型号".concat(model).concat("正在被使用");
        }

        StringBuilder builder1 = new StringBuilder();
        builder1.append("select count(0) from t_pack where cell_model='").append(model).append("' ");
        log.info("SQL: {}", builder1.toString());
        Query q1 = this.getQueryBySQL(builder1.toString(), new HashMap<>());
        amount = ((BigInteger) q1.getSingleResult()).longValue();
        if (amount > 0) {
            return "单体型号".concat(model).concat("正在被使用");
        }
        return OK;
    }

    /**
     * 校验单体型号是否已经存在
     *
     * @param dto 参数
     * @return java.lang.String
     */
    private String checkCellModelExist(CellDto dto) {
        StringBuilder builder = new StringBuilder();
        builder.append("select count(0) from t_cell where cell_model='").append(dto.getCellModel()).append("' ");
        if (StringUtils.isNotBlank(dto.getId())) {
            builder.append(" and id <> '").append(dto.getId()).append("'");
        }
        log.info("SQL: {}", builder.toString());
        Query q = this.getQueryBySQL(builder.toString(), new HashMap<>());
        long amount = ((BigInteger) q.getSingleResult()).longValue();
        if (amount > 0) {
            return "单体型号".concat(dto.getCellModel()).concat("已存在");
        }
        return OK;
    }

    /**
     * 校验包型号是否符合规则
     * @param model 参数
     * @return java.lang.String
     */
    private String checkModelRule(String model){
        if (model.length()!=7){
            return "包型号长度必须是7";
        }
        StringBuilder builder = new StringBuilder();
        builder.append("select count(0) from product_type")
                .append(" where status=1 and sn_type_en=3 and sn_type_code='")
                .append(model.substring(0, 2)).append("'");
        Query q = this.getQueryBySQL(builder.toString(), new HashMap<>());
        long amount = ((BigInteger) q.getSingleResult()).longValue();
        if (amount <= 0) {
            return "单体型号-电芯厂家编码错误";
        }
        StringBuilder builder1 = new StringBuilder();
        builder1.append("select count(0) from product_type")
                .append(" where status=1 and sn_type_en=4 and sn_type_code='")
                .append(model.substring(2, 3)).append("'");
        q = this.getQueryBySQL(builder1.toString(), new HashMap<>());
        amount = ((BigInteger) q.getSingleResult()).longValue();
        if (amount <= 0) {
            return "单体型号-电芯类型编码错误";
        }
        StringBuilder builder2 = new StringBuilder();
        builder2.append("select count(0) from product_type")
                .append(" where status=1 and sn_type_en=6 and sn_type_code= CONVERT('")
                .append(model.substring(3, 6)).append("',UNSIGNED)");
        q = this.getQueryBySQL(builder2.toString(), new HashMap<>());
        amount = ((BigInteger) q.getSingleResult()).longValue();
        if (amount <= 0) {
            return "单体型号-电芯容量编码错误";
        }
        if(!(Pattern.matches("[A-Z]{1}",model.substring(6)))) {
            return "单体型号编号不符合规则,最后一位,必须为A-Z字母";
        }
        if(!(Pattern.matches("[^IO]{1}",model.substring(6)))) {
            return "单体型号编号不符合规则，最后一位不能包含I或者O";
        }

        return OK;
    }
}
